﻿using Microsoft.Extensions.DependencyInjection;
using XF.Domain.Dependency;

namespace DataSharing.DataExchange
{
    public class DataExchangeClient : ISingletonDependency, ISelfDependency
    {
        private readonly IServiceScopeFactory _scopeFactory;
        public DataExchangeClient(IServiceScopeFactory scopeFactory)
        {
            _scopeFactory = scopeFactory;
        }

        /// <summary>
        /// 
        /// </summary>
        /// <typeparam name="TResponse"></typeparam>
        /// <param name="url"></param>
        /// <param name="httpMethod"></param>
        /// <param name="stringContent"></param>
        /// <param name="cancellationToken"></param>
        /// <returns></returns>
        public async Task<TResponse?> RequestAsync<TResponse>(string url, string httpMethod, string? stringContent = null, CancellationToken cancellationToken = default)
        {
            using var scope = _scopeFactory.CreateScope();
            var provider = scope.ServiceProvider;
            var channelconfigManager = provider.GetRequiredService<IChannelConfigurationManager>();
            var httpInvoker = provider.GetRequiredService<IHttpInvoker>();
            var configDataExchange = channelconfigManager.GetConfigurationCityHandOver();
            return await httpInvoker.RequestStringContentAsync<TResponse>(url, httpMethod, stringContent,
                d => d.SetHttpClient(configDataExchange.BaseUrl), cancellationToken: cancellationToken);
        }

    }
}
