﻿using DataSharing.DataExchange;
using DataSharing.Enterprise;
using DataSharing.Province;
using Microsoft.Extensions.Logging;
using Quartz;
using XF.Domain.Queues;

namespace DataSharing.SendTask
{
    public class TaskExecuteJob : IJob, IDisposable
    {
        private readonly IQueue _queue;
        private readonly ILogger<TaskExecuteJob> _logger;
        private readonly IPusherProviderService _pusherProviderService;
        private readonly IEnterprisePusherProviderService _enterprisePusherProviderService;
        private readonly IDataExchangePusherProviderService _dataExchangePusherProviderService;

        /// <summary>
        /// 
        /// </summary>
        /// <param name="queue"></param>
        /// <param name="logger"></param>
        /// <param name="pusherProviderService"></param>
        /// <param name="enterprisePusherProviderService"></param>
        /// <param name="dataExchangePusherProviderService"></param>
        public TaskExecuteJob(IQueue queue, ILogger<TaskExecuteJob> logger, IPusherProviderService pusherProviderService,
         IEnterprisePusherProviderService enterprisePusherProviderService,
         IDataExchangePusherProviderService dataExchangePusherProviderService)
        {
            _queue = queue;
            _logger = logger;
            _pusherProviderService = pusherProviderService;
            _enterprisePusherProviderService = enterprisePusherProviderService;
            _dataExchangePusherProviderService = dataExchangePusherProviderService;
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="context"></param>
        /// <returns></returns>
        public async Task Execute(IJobExecutionContext context)
        {
            var task = await _queue.DequeueAsync<DsSendTask>(SendTaskDefaults.SendTaskQueue, context.CancellationToken);
            if (task is not null)
            {
                switch (task.PlatformSource)
                {
                    case Share.Enums.EPlatformSource.Province:
                        await _pusherProviderService.SendProvinceDataPusher(task, context.CancellationToken);
                        break;
                    case Share.Enums.EPlatformSource.Police110:
                        break;
                    case Share.Enums.EPlatformSource.CityDataExchange:
                        await _dataExchangePusherProviderService.SendDataExchangeDataPusher(task, context.CancellationToken);
                        break;
                    case Share.Enums.EPlatformSource.ConvergenceMedia:
                        break;
                    case Share.Enums.EPlatformSource.Enterprise:
                        await _enterprisePusherProviderService.SendEnterpriseDataPusher(task, context.CancellationToken);
                        break;
                    default:
                        break;
                }
                //await _sendTaskDataService.SendTask(task, context.CancellationToken);
                _logger.LogInformation($"send task execute, taskId: {task.Id}");
            }
        }

        /// <summary>
        /// 
        /// </summary>
        public void Dispose()
        {
            _logger.LogInformation($"execute job disposing");
        }
    }
}